Package: npGSEA
Type: Package
Title: Permutation approximation methods for gene set enrichment
        analysis (non-permutation GSEA)
Version: 1.49.0
Date: 2015-5-6
Author: Jessica Larson and Art Owen
Maintainer: Jessica Larson <larson.jess@gmail.com>
Imports: Biobase, methods, BiocGenerics, graphics, stats
Suggests: ALL, genefilter, limma, hgu95av2.db, ReportingTools,
        BiocStyle
Depends: GSEABase (>= 1.24.0)
Description: Current gene set enrichment methods rely upon permutations
        for inference.  These approaches are computationally expensive
        and have minimum achievable p-values based on the number of
        permutations, not on the actual observed statistics.  We have
        derived three parametric approximations to the permutation
        distributions of two gene set enrichment test statistics.  We
        are able to reduce the computational burden and granularity
        issues of permutation testing with our method, which is
        implemented in this package. npGSEA calculates gene set
        enrichment statistics and p-values without the computational
        cost of permutations.  It is applicable in settings where one
        or many gene sets are of interest.  There are also built-in
        plotting functions to help users visualize results.
License: Artistic-2.0
biocViews: GeneSetEnrichment, Microarray, StatisticalMethod, Pathways
Collate: 'getIncidence.R' 'miscFunctions.R' 'miscDataPrepFunctions.R'
        'npGSEA.R' 'AllClasses.R' 'AllGenerics.R'
        'npGSEAResultBeta-accessors.R' 'npGSEAResultChiSq-accessors.R'
        'npGSEAResultNorm-accessors.R' 'npGSEAPlot-methods.R'
        'show-methods.R' 'pValues-methods.R'
Packaged: 2026-04-29 00:06:52 UTC; biocbuild
git_url: https://git.bioconductor.org/packages/npGSEA
git_branch: devel
git_last_commit: ec632c0
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-04-28
NeedsCompilation: no

importClassesFrom(methods, character, list, matrix, numeric)

importMethodsFrom(Biobase, exprs)
importMethodsFrom(BiocGenerics, lapply, mapply, match, ncol, rownames,
    sort, unlist)
importMethodsFrom(methods, show)

importFrom(graphics, abline, plot)
importFrom(methods, is, new)
importFrom(stats, dbeta, dchisq, dnorm, lm, pbeta, pchisq, pnorm,
    resid)

import(GSEABase)

export(getIncidence)
export(npGSEA)

exportClasses(npGSEAResultNorm, npGSEAResultNormCollection,
    npGSEAResultBeta, npGSEAResultBetaCollection,
    npGSEAResultChiSq, npGSEAResultChiSqCollection)

exportMethods(geneSetName, stat, sigmaSq, pValues, pTwoSided, pLeft, 
    pRight, xSet, npGSEAPlot, zStat, betaStat, alphaValue, 
    betaValue, chiSqStat, DF, show, betaHats)

runNormApprox <- function(xg, y, wg, set){
    betahatG <- .calcBetaHatG (xg, y)
    ThatGw <- .calcThatGw (betahatG, wg)
    varThatGw <- .calcVarThatGw (xg, y, wg)
    stats <- .calcPvaluesNorm (ThatGw, varThatGw)
    return( new("npGSEAResultNorm", 
        geneSetName = setName(set),
        zStat = as.numeric(ThatGw/sqrt(varThatGw)),
        ThatGw = as.numeric(ThatGw),
        varThatGw = as.numeric(varThatGw), 
        pLeft = as.numeric(stats$pLeft), 
        pRight = as.numeric(stats$pRight), 
        pTwoSided = as.numeric(stats$pTwoSided), 
        xSet = xg,
        betaHats = betahatG) )
}

.calcBetaHatG <- function(xg, y){
	nobs <- length(y)
    ###a vector containing each Betahat_g
   	betahat_g <- (xg %*% y)/nobs  
   	return(betahat_g)
}

##test stat for gene set in experiment (norm approx):
.calcThatGw <- function(betahat_g, wg) {
    ##sum of all the Betahat_g's wtd by  their corresponding weights
    ThatGw <- wg %*% betahat_g 
    return(ThatGw)
}

##calculates variance of permuted ThatGw (norm approx)
## based on eq 5, section 3.3 in Larson and Owen 2014
.calcVarThatGw <- function(xg, y, wg) {
    nobs <- length(y)
    XGi <- wg %*% xg 
    XbarGG <- mean(XGi^2)
    mu2 <- mean(y^2)  ##section 3.1
    VarThatGw  <- (mu2/(nobs-1))*XbarGG
    return(VarThatGw)
}


##calculates p-values for our appoximation (norm)
.calcPvaluesNorm <- function(ThatGw, varThatGw){
    Tscaled <- ThatGw/sqrt(varThatGw)
    pL <- pnorm(Tscaled)
    pR <- pnorm(-Tscaled)
    pC <- 2*min(pL, pR)
    return(list(pLeft=pL, pRight=pR, pTwoSided=pC))
}

### calc Beta dist'n based stats
##uses many of the same functions as norm approx
runBetaApprox <- function(xg, y, wg, set, epAdj){    
    #first we get betahats, That and its variance
    betahatG <- .calcBetaHatG (xg, y)
    ThatGw <- .calcThatGw (betahatG, wg)
    varThatGw <- .calcVarThatGw(xg, y, wg)

    ##next we calculate A and B, see section 3.3
    nobs <- length(y)
    XGi <- wg %*% xg 
    sortedXGi <- sort(XGi)
    sortedY <- sort(y)
    sortedYdecreasing <- sort(y, decreasing=TRUE)
    A <- (sortedXGi%*%sortedYdecreasing)/nobs 
    B <- (sortedXGi%*%sortedY)/nobs

    ##next we calculate alpha and beta and our betaStat, see eq. 3
    alpha <- (A/(B-A))*(A*B/varThatGw+1)
    beta <- (-B/(B-A))*(A*B/varThatGw+1)
    betaStat <- (ThatGw-A)/(B-A)
    pvals <- .calcPvaluesBeta (betaStat, alpha, beta, y, epAdj)
    
    return(new("npGSEAResultBeta",
        geneSetName = setName(set),
        betaStat = as.numeric(betaStat),
        ThatGw = as.numeric(ThatGw),
        varThatGw = as.numeric(varThatGw), 
        alpha = as.numeric(alpha), 
        beta = as.numeric(beta), 
        pLeft = as.numeric(pvals$pLeft), 
        pRight = as.numeric(pvals$pRight), 
        pTwoSided = as.numeric(pvals$pTwoSided), 
        xSet = xg,
        betaHats = betahatG)  )
}

##calculates p-values for our appoximation (beta)
##with the epsilon min p-value based on the number of levels of y
.calcPvaluesBeta <- function(betaStat, alpha, beta, y, epAdj){
	epsilon <- 0
	
	if (epAdj==TRUE){
		yFactor <- as.factor(y)
    	yLevels <- levels(yFactor)
   	 	numLevels <- length(yLevels)
    	n <- length(y)
    	epsilon <- 1
    	for (j in 1:numLevels){
    		locLevel <- which(yFactor==yLevels[j])
    		nj <- length(locLevel)
    		epsilon <- epsilon*factorial(nj)
    	}
    	epsilon <- epsilon/factorial(n)
	}
	
	pL <- pbeta(betaStat, alpha, beta)
	pLeft <- epsilon + (1-2*epsilon)*pL
	
    pR <- pbeta(betaStat, alpha, beta, lower.tail=FALSE)
    pRight <- epsilon + (1-2*epsilon)*pR
    
    pC <- 2*min(pLeft, pRight)
    return(list(pLeft=pLeft, pRight=pRight, pTwoSided=pC) )
}




##test stat for gene set in experiment (chisq approx):
runChisqApprox <- function(xg, y, wg, set){
    if( length(y) < 4 ){
        stop("Number of samples is too small 
        for permutation approximation; 
        you need at least 4 samples for the 
        chiSq approximation analysis")
    }
    #Get set statistics
    betahatG <- .calcBetaHatG (xg, y)
    ChatGw <- .calcChatGw(betahatG, wg)
    # Get moment approximation to permutation distribution
    chiSqMoments <- .calcChiSqMoments(xg, y, wg)   
    df <- as.numeric(2*chiSqMoments$mu^2/chiSqMoments$var)
    sigsq <- as.numeric(chiSqMoments$mu/df)
    pQ <- as.numeric(1 - pchisq(ChatGw/sigsq, df) )
   
	##check moments and NaN's
	if (is.nan(pQ)==TRUE){
		print(paste0("The moments for the chisquare approximation (mu= ", chiSqMoments$mu , " and var = ",
		chiSqMoments$var, "), which did not lead to a valid approximation.  This may be a result of a small sample size at one or more levels 
		of the predictor or response variable"))
	}
    
    return(new("npGSEAResultChiSq",
        geneSetName = setName(set),
        chiSqStat = ChatGw/sigsq,
        ChatGw = ChatGw,
        sigmaSq = sigsq, 
        DF= df, 
        pTwoSided = pQ, 
        xSet = xg,
        betaHats = betahatG) )
}

# sum of squared dot products, genes x response y with wts
.calcChatGw <- function(betahat_g, wg) {
    C_Gw <- wg %*%(betahat_g)^2
    return(as.numeric(C_Gw))
}

# Return permutation based moments (chisq approx)
##section 3.1, eq4 of paper
.calcChiSqMoments <- function(xg, y, wg){
    x <- t(xg)
    nobs <- length(y)  ##num of obs
    G = ncol(x)  ##number of genes in G

    ##get A and B matrices
    A <- .getA(nobs)
    B <- .getB()
    AtB = t(A) %*% B  ###2x2 matrix of constants

    ## Precompute X moments
    xgh = xgghh = matrix(NA,G,G)
    for( g in 1:G ){
        for( h in 1:g ){
            xgh[g,h]   = xgh[h,g]   = mean( x[,g] * x[,h] )
            xgghh[g,h] = xgghh[h,g] = mean( x[,g]^2 * x[,h]^2 )
        }
    }

    ## Precompute Y moments
    ymoments <- matrix( 0,2,1 )
    mu2 <- mean(y^2)
    ymoments[1,1] <- mu2^2  
    ymoments[2,1] <- mean(y^4)  #mu4

    ##Calculate mean square (lemma 1)
    meansquare <- rep(0,G)
    for( g in 1:G ){
        meansquare[g] <- mu2 * xgh[g,g]/(nobs-1)
    }

    ##Calculate cov square (lemma 2, collary 2)
    covsquare <- matrix(0,G,G)
    xmoments <- matrix( 0,2,1 )
    for( g in 1:G ){
        for( h in 1:g ){
            xmoments[1,1]  <- (xgh[g,g]*xgh[h,h]+2*xgh[g,h]^2)/nobs^2
            xmoments[2,1]  <- xgghh[g,h]/nobs^3
            covsquare[g,h] = covsquare[h,g] = 
                t(ymoments) %*% AtB %*% xmoments - 
                ymoments[1,1] * xgh[g,g] * xgh[h,h] /(nobs-1)^2
        }
    }
  
    ##Calculate E(C_tildeGw) and var(C_tildeGw) (eq 4, section 3.1)
    mu <- wg%*% meansquare 
    var <- (wg%*% covsquare)%*%wg
    return( list(mu=mu, var = var) )
}

## Calculate matrix A from Lemma 2, section 3.1
.getA <- function(n){
    A = matrix(0, 5, 2)
    A[,1] = c( 0, 0, n/(n-1),-n/((n-1)*(n-2)), 3*n/((n-1)*(n-2)*(n-3)) )
    A[,2] = c(1,-1/(n-1),-1/(n-1),2/((n-1)*(n-2)),-6/((n-1)*(n-2)*(n-3)))
    return(A)
}
## Calculate matrix B from Lemma 2, section 3.1
.getB <- function(){
    B <- matrix(0, 5, 2)
    B[,1] <- c(0, 0, 1, -2, 1)
    B[,2] <- c(1, -4, -3, 12, 6)
    return(B)
}

setMethod("geneSetName", "npGSEAResultNorm", 
    function(object){object@geneSetName}
)
setMethod("geneSetName", "npGSEAResultNormCollection", 
    function(object){lapply(object, function(y) y@geneSetName)}
)

setMethod("stat", "npGSEAResultNorm", 
    function(object){object@ThatGw}
)
setMethod("stat", "npGSEAResultNormCollection", 
    function(object){lapply(object, function(y) y@ThatGw)}
)

setMethod("sigmaSq", "npGSEAResultNorm", 
    function(object){object@varThatGw}
)
setMethod("sigmaSq", "npGSEAResultNormCollection", 
    function(object){lapply(object, function(y) y@varThatGw)}
)

setMethod("pTwoSided", "npGSEAResultNorm", 
    function(object){object@pTwoSided}
)
setMethod("pTwoSided", "npGSEAResultNormCollection", 
    function(object){lapply(object, function(y) y@pTwoSided)}
)

setMethod("pLeft", "npGSEAResultNorm", 
    function(object){object@pLeft}
)
setMethod("pLeft", "npGSEAResultNormCollection", 
    function(object){lapply(object, function(y) y@pLeft)}
)

setMethod("pRight", "npGSEAResultNorm", 
    function(object){object@pRight}
)
setMethod("pRight", "npGSEAResultNormCollection", 
    function(object){lapply(object, function(y) y@pRight)}
)

setMethod("xSet", "npGSEAResultNorm", 
    function(object){object@xSet}
)
setMethod("xSet", "npGSEAResultNormCollection", 
    function(object){lapply(object, function(y) y@xSet)}
)

setMethod("betaHats", "npGSEAResultNorm", 
    function(object){object@betaHats}
)
setMethod("betaHats", "npGSEAResultNormCollection", 
    function(object){lapply(object, function(y) y@betaHats)}
)

setMethod("zStat", "npGSEAResultNorm", 
    function(object){object@zStat}
)
setMethod("zStat", "npGSEAResultNormCollection", 
    function(object){lapply(object, function(y) y@zStat)}
)

##' Calculates the incidence of a gene set in an experiment 
##'
##' getIncidence returns an incidence vector of the location of the genes within a gene set in a list of genes in an experiment and vise-versa.
##' @title Determines the incidence of a gene set in a list of genes. 
##' @param universeIDs A vector containing the list of possible gene ids in the universe (or experiment).
##' @param set A GeneSet object containing a set of genes of interest
##' @return a list of inSet and inExp.  inSet is a vector with the same length as universeIDs.  Each value of inSet is 1 if the gene is in the set and 0 otherwise.  inExp is a vector with the same length as geneIds(set), the number of genes in the set.  Each value of inExp is 1 if the gene is in universeIDs and 0 otherwise.
##' @author Jessica L. Larson and Art Owen
##' @export
getIncidence <- function(universeIDs, set) {
    ##check if gene set
    if(is(set)[1] != "GeneSet" )
        {stop("set must be a GeneSet object") }     
    geneSetIds <- unlist(geneIds(set))
    inSet <- rep(0, length(universeIDs))
    inSet[match(geneSetIds, universeIDs)] <- 1
    inExp <- rep(0, length(geneSetIds))
    inExp[match(universeIDs, geneSetIds)] <- 1
    output <- list(inSet=inSet, inExp=inExp)
    output
}

## show 
setMethod("show", "npGSEAResultNorm", 
    function(object){
        cat(paste0( "Normal Approximation for ", object@geneSetName, "\n"))
        cat(paste0("T_Gw = ", signif(object@ThatGw,3), "\n") )
        cat(paste0("var(T_Gw) = ", signif(object@varThatGw,3), "\n") )
        cat(paste0("pLeft = ", signif(object@pLeft,3), 
                ", pRight = ", signif(object@pRight,3), 
                ", pTwoSided = ",  signif(object@pTwoSided,3), "\n") )
    }
)
setMethod("show", "npGSEAResultBeta", 
    function(object){
        cat(paste0( "Beta Approximation for ", object@geneSetName, "\n"))
        cat(paste0("T_Gw = ", signif(object@ThatGw,3), "\n"))
        cat(paste0("var(T_Gw) = ", signif(object@varThatGw,3), "\n"))
        cat(paste0("pLeft = ", signif(object@pLeft,3), 
                ", pRight = ", signif(object@pRight,3), 
                ", pTwoSided = ",  signif(object@pTwoSided,3), "\n"))
    }
)

setMethod("show", "npGSEAResultChiSq",
    function(object) {
        cat (paste0( "Chi-sq Approximation for ", object@geneSetName), "\n" )
        cat (paste0("C_Gw = ", signif(object@ChatGw,3), "\n"))
        cat (paste0("df = ", signif(object@DF,3), 
            ", sigmaSq = ", signif(object@sigmaSq,3), "\n") )
        cat (paste0("pTwoSided = ",  signif(object@pTwoSided,3), "\n") )
    }
)

setMethod("geneSetName", "npGSEAResultBeta", 
    function(object){object@geneSetName}
)
setMethod("geneSetName", "npGSEAResultBetaCollection", 
    function(object){lapply(object, function(y) y@geneSetName)}
)

setMethod("stat", "npGSEAResultBeta", 
    function(object){object@ThatGw}
)
setMethod("stat", "npGSEAResultBetaCollection", 
    function(object){lapply(object, function(y) y@ThatGw)}
)

setMethod("sigmaSq", "npGSEAResultBeta", 
    function(object){object@varThatGw}
)
setMethod("sigmaSq", "npGSEAResultBetaCollection", 
    function(object){lapply(object, function(y) y@sigmaSq)}
)

setMethod("pTwoSided", "npGSEAResultBeta", 
    function(object){object@pTwoSided}
)
setMethod("pTwoSided", "npGSEAResultBetaCollection", 
    function(object){lapply(object, function(y) y@pTwoSided)}
)

setMethod("pLeft", "npGSEAResultBeta", 
    function(object){object@pLeft}
)
setMethod("pLeft", "npGSEAResultBetaCollection", 
    function(object){lapply(object, function(y) y@pLeft)}
)

setMethod("pRight", "npGSEAResultBeta", 
    function(object){object@pRight}
)
setMethod("pRight", "npGSEAResultBetaCollection", 
    function(object){lapply(object, function(y) y@pRight)}
)

setMethod("xSet", "npGSEAResultBeta", 
    function(object){object@xSet}
)
setMethod("xSet", "npGSEAResultBetaCollection", 
    function(object){lapply(object, function(y) y@xSet)}
)

setMethod("betaHats", "npGSEAResultBeta", 
    function(object){object@betaHats}
)
setMethod("betaHats", "npGSEAResultBetaCollection", 
    function(object){lapply(object, function(y) y@betaHats)}
)

setMethod("betaStat", "npGSEAResultBeta", 
    function(object){object@betaStat}
)
setMethod("betaStat", "npGSEAResultBetaCollection", 
    function(object){lapply(object, function(y) y@betaStat)}
)

setMethod("alphaValue", "npGSEAResultBeta", 
    function(object){object@alpha}
)
setMethod("alphaValue", "npGSEAResultBetaCollection", 
    function(object){lapply(object, function(y) y@alpha)}
)

setMethod("betaValue", "npGSEAResultBeta", 
    function(object){object@beta}
)
setMethod("betaValue", "npGSEAResultBetaCollection", 
    function(object){lapply(object, function(y) y@beta)}
)

setClass("npGSEAResultNorm",
    representation(
    geneSetName = "character",
    zStat = "numeric",
    ThatGw = "numeric",
    varThatGw = "numeric",
    pLeft="numeric",
    pRight="numeric",
    pTwoSided="numeric",
    xSet = "matrix",
    betaHats = "matrix")
)

setClass("npGSEAResultNormCollection",
    contains="list",
    validity = function(object) {
        msg <- NULL
        if (!all(sapply(object, is, "npGSEAResultNorm")))
            msg <- c(msg, 
                "members must all be 'npGSEAResultNorm' classes")
        if (!is.null(msg))
            msg
        else
            TRUE
    }
)

setClass("npGSEAResultBeta",
    representation(
    geneSetName = "character",
    betaStat="numeric",
    ThatGw = "numeric",
    varThatGw = "numeric",
    alpha="numeric",
    beta="numeric",
    pLeft="numeric",
    pRight="numeric",
    pTwoSided="numeric",
    xSet="matrix",
    betaHats = "matrix")
)

setClass("npGSEAResultBetaCollection",
    contains="list",
    validity = function(object) {
        msg <- NULL
        if (!all(sapply(object, is, "npGSEAResultBeta")))
            msg <- c(msg, 
                "members must all be 'npGSEAResultBeta' classes")
        if (!is.null(msg))
            msg
        else
            TRUE
         }
)

setClass("npGSEAResultChiSq",
    representation(
    geneSetName = "character",
    chiSqStat="numeric",
    ChatGw="numeric",
    sigmaSq="numeric",
    DF="numeric",
    pTwoSided="numeric",
    xSet="matrix",
    betaHats = "matrix")
)

setClass("npGSEAResultChiSqCollection",
    contains="list",
    validity = function(object) {
        msg <- NULL
        if (!all(sapply(object, is, "npGSEAResultChiSq")))
            msg <- c(msg, 
                "members must all be 'npGSEAResultChiSq' classes")
        if (!is.null(msg))
            msg
        else
            TRUE
    }
)

setMethod("geneSetName", "npGSEAResultChiSq", 
    function(object){object@geneSetName}
)
setMethod("geneSetName", "npGSEAResultChiSqCollection", 
    function(object){lapply(object, function(y) y@geneSetName)}
)

setMethod("stat", "npGSEAResultChiSq", 
    function(object){object@ChatGw}
)
setMethod("stat", "npGSEAResultChiSqCollection", 
    function(object){lapply(object, function(y) y@ChatGw)}
)

setMethod("sigmaSq", "npGSEAResultChiSq", 
    function(object){object@sigmaSq}
)
setMethod("sigmaSq", "npGSEAResultChiSqCollection", 
    function(object){lapply(object, function(y) y@sigmaSq)}
)

setMethod("pTwoSided", "npGSEAResultChiSq", 
    function(object){object@pTwoSided}
)
setMethod("pTwoSided", "npGSEAResultChiSqCollection", 
    function(object){lapply(object, function(y) y@pTwoSided)}
)

setMethod("xSet", "npGSEAResultChiSq", 
    function(object){object@xSet}
)
setMethod("xSet", "npGSEAResultChiSqCollection", 
    function(object){lapply(object, function(y) y@xSet)}
)

setMethod("betaHats", "npGSEAResultChiSq", 
    function(object){object@betaHats}
)
setMethod("betaHats", "npGSEAResultChiSqCollection", 
    function(object){lapply(object, function(y) y@betaHats)}
)

setMethod("chiSqStat", "npGSEAResultChiSq", 
    function(object){object@chiSqStat}
)
setMethod("chiSqStat", "npGSEAResultChiSqCollection", 
    function(object){lapply(object, function(y) y@chiSqStat)}
)

setMethod("DF", "npGSEAResultChiSq", 
    function(object){object@DF}
)
setMethod("DF", "npGSEAResultChiSqCollection", 
    function(object){lapply(object, function(y) y@DF)}
)

setMethod("npGSEAPlot", "npGSEAResultNorm", 
    function(object){
        xrange <- c(-4,4)
        obsZStat <- object@zStat
        if(obsZStat >= 4) {xrange <- c( -(obsZStat+1), (obsZStat+1) ) } 
        if(obsZStat <= -4) {xrange <- c( (obsZStat-1), -(obsZStat-1) ) }
        plot(function(x) dnorm(x),  
            main = paste0("Standard Normal Distribution"), 
            xlim=xrange, xlab="Z values", ylab="Density")
        abline(v = obsZStat, col="red")
    }
)

setMethod("npGSEAPlot", "npGSEAResultBeta", 
    function(object){
        xrange <- c(0,1)
        obsBStat <- object@betaStat
        alph <- object@alpha
        bet <- object@beta
        if(obsBStat >= 1) { xrange<- c( -(obsBStat+1),(obsBStat+1) ) }
        if(obsBStat <= 0) { xrange<- c( (obsBStat-1),-(obsBStat-1) ) }
        plot(function(x) dbeta(x, alph, bet), 
                main = paste0("Beta distribution with \n alpha = ", 
                    round(alph, 2), " and beta = ", round(bet,2)), 
                xlim = xrange, xlab = "Beta values", ylab = "Density")
        abline(v = obsBStat, col="red")
    }
)
setMethod("npGSEAPlot", "npGSEAResultChiSq", 
    function(object){
        xrange <- c(0,4)
        obsStat <- object@chiSqStat
        df <- object@DF
        if(obsStat >= 4) {xrange <- c(0, (obsStat+1) ) }
        plot(function(x) dchisq(x, df),  
            main = paste0("Chi-sq distribution with \n df=", 
                round(df, 2)), xlim=xrange, xlab="Chi-sq values", ylab="Density")
        abline(v = obsStat, col="red")
    }
)

 ##' Calculates an approximation of the permutation GSEA statistics and p-values
##'
##' This function calculates the permutation gene set enrichment analysis test statistic and p-value without actually running the permutation.  We account for the covariance among the genes within the set and approximate the corresponding permutation distribution.  For more details on the method see Larson and Owen (2014).
##' @title Runs the non-permutation GSEA 
##' @param x A matrix of expression data or an object of type ExpressionSet.  The columns of x represent samples in a given experiment.  The rows are genes.  The names of each row (or featureNames of the eSet) must be of the same type (e.g., entrez ids) as the ids of the gene set.
##' @param y A vector containing the treatment for each sample. The length of y must be more than 4 for the "chisq" approximation.  Each treatment group must have at least two observations for all approximation methods.
##' @param covars A vector or matrix containing covariate(s) of interest, optional
##' @param set A GeneSet object containing a set of genes of interest or a GeneSetCollection object containing a collection of GeneSets
##' @param approx A string of either "norm" (default), "beta" or "chiSq".  If "norm", the normal approximation to the non-permutation GSEA is calculated and returned.  If "beta", the beta approximation is reported.  If "chiSq", the Chi-squared approximation to the permutation GSEA is calculated.
##' @param w A vector or list containing the weights of each gene in the set or sets, optional.  If w is a list, the number of elements in the list must correspond to the number of gene sets in the collection. 
##' @param epsilonBetaAdj A boolean indicating whether or to not to use an epsilon adjusted p-value for the Beta approximation.  When TRUE, this prevents observed p-values of 0.  The default is TRUE.
##' @param scaleXY A boolean indicating whether or to not to scale x and y.  The default is TRUE.
##' @param uniVarX A boolean indicating whether or to not to scale x to have unit variance.  The default is TRUE.
##' @return an object with the corresponding GSEA results.  If approx="norm" an npGSEAResultNorm object is returned.  If approx="beta" a npGSEAResultBeta object is returned.  If approx="chiSq" a npGSEAResultChiSq object is returned.  If set is a GeneSetCollection (i.e., multiple sets of interest), then the corresponding npGSEAResultNormCollection, npGSEAResultBetaCollection, or npGSEAResultChiSqCollection is returned.
##' @author Jessica L. Larson and Art Owen
##' @export
npGSEA <- function(x, y, set, covars = NULL, approx = c("norm", "beta", "chiSq"), w = NULL, epsilonBetaAdj=TRUE, scaleXY=TRUE, uniVarX=TRUE){
    approx <- match.arg(approx, c("norm", "beta", "chiSq"), several.ok = FALSE)    
    if(is(x)[1]== "ExpressionSet") { x <- exprs(x) }
    if(dim(x) [2] != length(y) )
        {stop("Must have the same amount of samples in x and y") }
        
    ##scale y to sum to zero
    if(scaleXY==TRUE) { y <- .adjustY(y)}
      
    ######collection of sets:
    if (is(set, "GeneSetCollection") == TRUE ){
        if( (is.null(w) == FALSE) && ( is.list(w)== FALSE ) ){
            stop("You must provide weights for each gene set in 
            your collection as a list. 
            The length of w must be equal to the 
            length of your GeneSetCollection")
        }
        if( (is.null(w) == FALSE) && ( length(w)!=length(set) ) ){
            stop("You must provide weights for each gene 
            set in your collection.  
            The length of w is not equal to the 
            length of your GeneSetCollection")
        }
        ##make an empty list of weights if w is null
        if (is.null(w) ==TRUE )  { w <- vector("list", length(set)) }
        ##norm approx
        if (approx=="norm"){
            res <- mapply(function(singleSet, singleW){
            ##prep data
            xyz <- .prepXYZ(x, y,  covars, singleSet, scaleXY, uniVarX)
            xg <- xyz$xg
            y <- xyz$y
            wg <- .prepW(singleW, singleSet,  xyz$inset)
            ##run analysis
            runNormApprox(xg, y, wg, singleSet)            
            }, set, w )
            output <- new("npGSEAResultNormCollection", res)
        }
        ##beta approx
        if (approx=="beta"){              
            res <- mapply(function(singleSet, singleW){
            xyz <- .prepXYZ(x, y,  covars, singleSet, scaleXY, uniVarX)
            xg <- xyz$xg
            y <- xyz$y
            wg <- .prepW(singleW, singleSet,  xyz$inset)
            runBetaApprox(xg, y, wg, singleSet, epsilonBetaAdj)            
            }, set, w )
            output <- new("npGSEAResultBetaCollection", res)
        }
        ##chisq approx
        if (approx=="chiSq"){
            res <- mapply(function(singleSet, singleW){
            xyz <- .prepXYZ(x, y, covars, singleSet, scaleXY, uniVarX)
            xg <- xyz$xg
            y <- xyz$y
            wg <- .prepW(singleW, singleSet, xyz$inset)
            runChisqApprox(xg, y, wg, singleSet) 
            }, set, w )
            output <- new("npGSEAResultChiSqCollection", res)
        }
    }  	

    #########single gene sets:
    else if (is(set, "GeneSet") ==TRUE ) {
        ##prep data
        xyz <- .prepXYZ(x, y,  covars, set, scaleXY, uniVarX)
        xg <- xyz$xg
        y <- xyz$y
        wg <- .prepW(w, set, xyz$inset)
        ###run the analysis:
        if (approx=="norm"){
            output <- runNormApprox(xg, y, wg, set) 
        }	
        if (approx=="beta"){
            output <- runBetaApprox(xg, y, wg, set, epsilonBetaAdj) 
        }	
        if (approx=="chiSq"){
            output <- runChisqApprox(xg, y, wg, set) 
        }
    }
    output
}

### R code from vignette source 'npGSEA.Rnw'

###################################################
### code chunk number 1: style
###################################################
BiocStyle::latex()


###################################################
### code chunk number 2: loadLibrariesAndData
###################################################
library(ALL)
library(hgu95av2.db)
library(genefilter)
library(limma)
library(GSEABase)
library(npGSEA)

data(ALL)

ALL <- ALL[, ALL$mol.biol %in% c('NEG','BCR/ABL') &
     !is.na(ALL$sex)]
ALL$mol.biol <- factor(ALL$mol.biol, 
     levels = c('NEG', 'BCR/ABL'))
ALL <- featureFilter(ALL)


###################################################
### code chunk number 3: adjustNames
###################################################
featureNames(ALL) <- select(hgu95av2.db, featureNames(ALL), 
                            "ENTREZID", "PROBEID")$ENTREZID


###################################################
### code chunk number 4: MakeSets
###################################################
xData <- exprs(ALL)
geneEids <- rownames(xData)
set.seed(12345)
set1 <- GeneSet(geneIds=sample(geneEids,15, replace=FALSE), 
             setName="set1", 
             shortDescription="This is set1")
set2 <- GeneSet(geneIds=sample(geneEids,50, replace=FALSE), 
             setName="set2",  
             shortDescription="This is set2")
set3 <- GeneSet(geneIds=sample(geneEids,100, replace=FALSE), 
             setName="set3",  
             shortDescription="This is set3")
set4 <- GeneSet(geneIds=sample(geneEids,500, replace=FALSE), 
            setName="set4",  
            shortDescription="This is set4")


###################################################
### code chunk number 5: RunLimma2NewSets
###################################################
model <- model.matrix(~mol.biol, ALL)
fit  <-  eBayes(lmFit(ALL, model))
tt <- topTable(fit, coef=2, n=200)
ttUp <- tt[which(tt$logFC >0), ]
ttDown <- tt[which(tt$logFC <0), ]

set5 <- GeneSet(geneIds=rownames(ttUp)[1:20], 
        setName="set5",  
        shortDescription="This is a true set of the top 20 DE 
        genes with a positive fold change")
set6 <- GeneSet(geneIds=rownames(ttDown)[1:20], 
        setName="set6",  
        shortDescription="This is a true set of the top 20 DE genes 
        with a negative fold change")
set7 <- GeneSet(geneIds=c(rownames(ttUp)[1:10], rownames(ttDown)[1:10]), 
        setName="set7",  
        shortDescription="This is a true set of the top 10 DE genes 
        with a positive and a negative fold change")


###################################################
### code chunk number 6: GSC
###################################################
gsc <- GeneSetCollection( c(set1, set2, set3, set4, set5, set6, set7) )
gsc


###################################################
### code chunk number 7: runIt
###################################################
yFactor <- ALL$mol.biol
res1 <- npGSEA(x = ALL, y = yFactor, set = set1)  ##with the eset
res1

res2_exprs <- npGSEA(xData, ALL$mol.biol, gsc[[2]])  ##with the expression data
res2_exprs 


###################################################
### code chunk number 8: accessorFunctions
###################################################
res3 <- npGSEA(ALL, yFactor, set3)  
res3
geneSetName(res3)
stat(res3)
sigmaSq(res3)
zStat(res3)
pTwoSided(res3)
pLeft(res3)
pValues(res3)
dim(xSet(res3))


###################################################
### code chunk number 9: plot3
###################################################
npGSEAPlot(res3)


###################################################
### code chunk number 10: runItNorm
###################################################
res5_norm <- npGSEA(ALL, yFactor, set5, approx= "norm")  
res5_norm
betaHats(res5_norm)
npGSEAPlot(res5_norm)


###################################################
### code chunk number 11: runItBeta
###################################################
res5_beta <- npGSEA(ALL, yFactor, set5, approx= "beta")
res5_beta
betaHats(res5_beta)
npGSEAPlot(res5_beta)


###################################################
### code chunk number 12: runItChiSq
###################################################
res5_chiSq <- npGSEA(ALL, yFactor, set5, approx= "chiSq")  
res5_chiSq
betaHats(res5_chiSq)
chiSqStat(res5_chiSq)
stat(res5_chiSq)
stat(res5_chiSq)/sigmaSq(res5_chiSq)
npGSEAPlot(res5_chiSq)


###################################################
### code chunk number 13: runItwts
###################################################
res7_nowts <- npGSEA(x = ALL, y= yFactor, set = set7)  
res7_nowts

wts <- apply(exprs(ALL)[match(geneIds(set7), featureNames(ALL)), ], 
						1, var)	
wts <- 1/wts				
res7_wts <- npGSEA(x = ALL, y = yFactor, set = set7,  w = wts, approx= "norm")  
res7_wts						


###################################################
### code chunk number 14: addCovariates
###################################################
res3_age <- npGSEA(x = ALL, y = yFactor, set = set3, covars = ALL$age) 
res3_age

res3_agesex <- npGSEA(x = ALL, y = yFactor, set = set3, covars = cbind(ALL$age, ALL$sex)) 
res3_agesex


###################################################
### code chunk number 15: runMultiplenpGSEA
###################################################
resgsc_norm <- npGSEA(x = ALL, y = yFactor, set = gsc)  
unlist( pLeft(resgsc_norm) )
unlist( stat (resgsc_norm) )
unlist( zStat (resgsc_norm) )


###################################################
### code chunk number 16: publishMultiplenpGSEA (eval = FALSE)
###################################################
## pvals <- p.adjust( unlist(pTwoSided(resgsc_norm)), method= "BH" )
## library(ReportingTools)
## npgseaReport <- HTMLReport (shortName = "npGSEA",
##         title = "npGSEA Results", reportDirectory = "./reports")
## publish(gsc, npgseaReport, annotation.db = "org.Hs.eg",
##         setStats = unlist(zStat (resgsc_norm)), setPValues = pvals)
## finish(npgseaReport)


###################################################
### code chunk number 17: sessInfo
###################################################
toLatex(sessionInfo())


###################################################
### code chunk number 18: resetOptions
###################################################
options(prompt="> ", continue="+ ")

setMethod("pValues", "npGSEAResultNorm", 
    function(object){
        cat(paste0("pLeft = ", signif(object@pLeft,3), 
                ", pRight = ", signif(object@pRight,3), 
                ", pTwoSided = ",  signif(object@pTwoSided,3), "\n") )
    }
)

setMethod("pValues", "npGSEAResultBeta", 
    function(object){
        cat(paste0("pLeft = ", signif(object@pLeft,3), 
                ", pRight = ", signif(object@pRight,3), 
                ", pTwoSided = ",  signif(object@pTwoSided,3), "\n"))
    }
)

setMethod("pValues", "npGSEAResultChiSq",
    function(object) {
        cat (paste0("pTwoSided = ",  signif(object@pTwoSided,3), "\n") )
        }
)

.prepXYZ <- function(x, y,  z=NULL, set, scaleXY=TRUE, uniVarX=TRUE) {
    ##get locale of set genes in experiment
    inset <- .locGenes(x, set)
    ##select out genes in set
    xset <- x[which(inset==1),]  ##rows are genes
    ##scale x to sum to zero
    xg <- xset
    ##center at 0
    if(scaleXY==TRUE) {xg <- .center(xset)}
    ##unit variance for X
    if(uniVarX==TRUE) {  
    	sdx <- sd(xg, na.rm=TRUE)
    	xg <- xg/sdx
    }
    
    .checkxg(xg)

    ##adjust for covariates for each gene and sample, if necessary
    ##z is our covars
   if(is.null(z) == FALSE ){
        gnames <- rownames(xg)
        xg <- t(apply(xg, 1, .getResids, z))
        rownames(xg) <- gnames
        y <- .getResids(y, z)
    }   
    return(list(xg=xg, y=y, inset=inset))
}

##check xg so that there is an error if 
##X_G takes only one value, 
##or if X_G takes only two values but one or both of those values appears only 1 time
.checkxg <- function(x){
	XGi <- apply(x, 2, sum)  ##sum the col'ns
	uniqueXG <- unique(XGi)	
    numVals = length(uniqueXG)
    if (numVals == 1){
    	stop( "You need at least two different values (across all samples) for the sum of the Xg's (for all g in G).") 
    }
    if (numVals == 2){
    	locVal1 = which(XGi==uniqueXG[1])
    	if (length(locVal1)==1){
    		stop( "If the sum of the Xg's (for all g in G) takes only two distinct values, then one of these values needs to appear more than once.")
    	}
    	locVal2 = which(XGi==uniqueXG[2])
    	if (length(locVal2)==1){
    		stop( "If the sum of the Xg's (for all g in G) takes only two distinct values, then one of these values needs to appear more than once.")
    	}
    }
}


##take into account any weights, or assign them all to 1, if none given
.prepW <- function(w, set, inset){
    if( (is.null(w) == FALSE) && ( length(w)!=sum(inset) ) ){
        stop( paste0 ("You must have the same number of weights as genes 
            in your set in this experiment.  ", 
            setName(set), " does not have the same number of 
            genes as there are 
            weights.\nRun getIncidence(rownames(x), set) 
            to determine which 
            genes are in this set and experiment." ) ) 
        }
    if (is.null(w) )  { w <- rep(1, sum(inset)) }
    return(w)
}

##get locale of set genes in experiment
##x is our x matrix (rows are genes, columns are samples)
##set is a GeneSet
.locGenes <- function(x, set) {
    incidence <- getIncidence(rownames(x), set)
    inset <- incidence$inSet
    if(sum(inset) == 0)
        {stop( paste0("No genes in ", setName(set), 
            " are in this experiment.  Make sure that the row names of x 
            are the same type as geneIds(set)") )
        }
    if(sum(inset) == 1)
        {stop( paste0("Only one gene in your set ", setName(set), 
            " is in this experiment.  Please choose a larger 
            set for this analysis"))
       }    
    return(inset)
}
        


##want sum(y)=0
##and want to make sure that there are at least two obs in each level of y
.adjustY <- function(y) {
	yFactor = as.factor(y)
    yLevels = levels(yFactor)
    numLevels=length(yLevels)
    if (numLevels < 3){
   	 	for (l in 1:numLevels){
    		locLevel = which(yFactor==yLevels[l])
    		if (length(locLevel)<2){
    			stop( paste0("Fewer than two observations in group ", yLevels[l], 
            " are in this experiment.  Make sure that there are at least two 
            observations in each treatment group if you only have two treatment groups.") ) 
    		}
    	}
	}
          
    y <- as.numeric(y)
    ynew <- y-mean(y)
    return(as.numeric(ynew))
}

# center rows of x around their mean
.center <- function(x) {
  xscaled <- scale(t(x), center=TRUE, scale=FALSE)
  return(t(xscaled))
}

##adjusts for covariates by taking residuals of fit
.getResids<-function(y1, x1) {
    return(resid(lm(y1~x1)))
}

setGeneric("geneSetName", function(object) 
                            standardGeneric("geneSetName"))

setGeneric("stat", function(object) standardGeneric("stat"))

setGeneric("sigmaSq", function(object) standardGeneric("sigmaSq"))

setGeneric("pValues", function(object) standardGeneric("pValues"))

setGeneric("pTwoSided", function(object) standardGeneric("pTwoSided"))

setGeneric("pLeft", function(object) standardGeneric("pLeft"))

setGeneric("pRight", function(object) standardGeneric("pRight"))

setGeneric("xSet", function(object) standardGeneric("xSet"))

setGeneric("betaHats", function(object) standardGeneric("betaHats"))

setGeneric("npGSEAPlot", function(object) 
                    standardGeneric("npGSEAPlot"))

setGeneric("zStat", function(object) standardGeneric("zStat"))

setGeneric("betaStat", function(object) standardGeneric("betaStat"))

setGeneric("alphaValue", function(object) standardGeneric("alphaValue"))

setGeneric("betaValue", function(object) standardGeneric("betaValue"))

setGeneric("chiSqStat", function(object) standardGeneric("chiSqStat"))

setGeneric("DF", function(object) standardGeneric("DF"))

%\VignetteIndexEntry{Running gene set enrichment analysis with the "npGSEA" package}
%\VignettePackage{npGSEA}
\documentclass[12pt]{article}


<<style, eval=TRUE, echo=FALSE, results=tex>>=
BiocStyle::latex()
@


\usepackage{cite}
\usepackage{Sweave}
\SweaveOpts{keep.source=TRUE,eps=FALSE,pdf=FALSE,png=TRUE,include=FALSE,width=4,height=4.5,resolution=150}
\setkeys{Gin}{width=0.5\textwidth}

% use a vertical rule for R output instead of prompts for R input
\usepackage{fancyvrb}
\definecolor{darkgray}{gray}{0.2}
\DefineVerbatimEnvironment{Sinput}{Verbatim}{xleftmargin=1em,formatcom={\color{darkgray}}}
\DefineVerbatimEnvironment{Soutput}{Verbatim}{xleftmargin=1em,frame=leftline,framerule=.6pt,rulecolor=\color{darkgray},framesep=1em,formatcom={\color{darkgray}}}
\fvset{listparameters={\setlength{\topsep}{0pt}}}
\renewenvironment{Schunk}{\vspace{\topsep}}{\vspace{\topsep}}



\author{Jessica L. Larson$^{1*}$ and Art Owen$^{2}$ \\[1em] 
    \small{$^{1}$ Department of Bioinformatics and Computational Biology, Genentech, Inc.} \\ 
    \small{$^{2}$ Department of Statistics, Stanford University} \\ 
    \small{\texttt{$^*$larson.jessica (at) gene.com}}}

\title{Moment based gene set enrichment testing -- the npGSEA package}

\date{\today}

\begin{document}

\maketitle
\tableofcontents
\newpage


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\section{Introduction }
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

Gene set methods are critical to the analysis of gene expression data. 
The \verb@npGSEA@ package provides methods to run 
permutation-based gene set enrichment analyses without 
the typically computationally expensive permutation cost.  These 
methods allow users to adjust for covariates and approximate 
corresponding permutation distributions.  We are currently evaluating 
the applicability and accuracy of our method for RNA-seq expression data.

Our methods
find the exact relevant moments of a weighted sum of (squared)
test statistics under permutation, taking into account correlations
among the test statistics.  We find moment-based gene set enrichment 
$p$-values that closely approximate the permutation method $p$-values.

This vignette describes a typical analysis workflow and includes some information 
about the statistical theory behind \verb@npGSEA@.  For more technical 
details, please see Larson and Owen, 2015 \nocite{larson:2015}.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\section{Example workflow for GSEA}
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\subsection{Preparing our gene sets and our dataset for analysis}
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
For our example, we will use the \verb@ALL@ dataset.  We begin by loading 
relevant libraries, subsetting the data,
and running \verb@featureFilter@ on this data set.  
For details on these methods, please see the \verb@limma@ manual.  

<<loadLibrariesAndData>>=
library(ALL)
library(hgu95av2.db)
library(genefilter)
library(limma)
library(GSEABase)
library(npGSEA)

data(ALL)

ALL <- ALL[, ALL$mol.biol %in% c('NEG','BCR/ABL') &
     !is.na(ALL$sex)]
ALL$mol.biol <- factor(ALL$mol.biol, 
     levels = c('NEG', 'BCR/ABL'))
ALL <- featureFilter(ALL)
@

We adjust the feature names of the \verb@ALL@ dataset so that they match the 
names of our gene sets below.  
We convert them to entrez ids.  
<<adjustNames>>=
featureNames(ALL) <- select(hgu95av2.db, featureNames(ALL), 
                            "ENTREZID", "PROBEID")$ENTREZID
@

We now make four arbitrary gene sets by randomly selecting 
from the genes in our universe.
<<MakeSets>>=
xData <- exprs(ALL)
geneEids <- rownames(xData)
set.seed(12345)
set1 <- GeneSet(geneIds=sample(geneEids,15, replace=FALSE), 
             setName="set1", 
             shortDescription="This is set1")
set2 <- GeneSet(geneIds=sample(geneEids,50, replace=FALSE), 
             setName="set2",  
             shortDescription="This is set2")
set3 <- GeneSet(geneIds=sample(geneEids,100, replace=FALSE), 
             setName="set3",  
             shortDescription="This is set3")
set4 <- GeneSet(geneIds=sample(geneEids,500, replace=FALSE), 
            setName="set4",  
            shortDescription="This is set4")
@

As a positive control, we also make three gene sets that include our 
top differentially expressed genes.
<<RunLimma2NewSets>>=
model <- model.matrix(~mol.biol, ALL)
fit  <-  eBayes(lmFit(ALL, model))
tt <- topTable(fit, coef=2, n=200)
ttUp <- tt[which(tt$logFC >0), ]
ttDown <- tt[which(tt$logFC <0), ]

set5 <- GeneSet(geneIds=rownames(ttUp)[1:20], 
        setName="set5",  
        shortDescription="This is a true set of the top 20 DE 
        genes with a positive fold change")
set6 <- GeneSet(geneIds=rownames(ttDown)[1:20], 
        setName="set6",  
        shortDescription="This is a true set of the top 20 DE genes 
        with a negative fold change")
set7 <- GeneSet(geneIds=c(rownames(ttUp)[1:10], rownames(ttDown)[1:10]), 
        setName="set7",  
        shortDescription="This is a true set of the top 10 DE genes 
        with a positive and a negative fold change")
@

We then collapse all of our gene sets into a \verb@GeneSetCollection@.  
For more information on
\verb@GeneSets@ and \verb@GeneSetCollections@, 
see the \verb@GSEABase@ manual.

<<GSC>>=
gsc <- GeneSetCollection( c(set1, set2, set3, set4, set5, set6, set7) )
gsc
@

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\subsection{Running npGSEA}
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Now that we have both our gene sets and experiment, we are ready to 
run \verb@npGSEA@ and 
determine the level of 
enrichment in our experiment.  We can use \verb@npGSEA@ with 
our \verb@eset@ or 
expression data (\verb@xData@) directly.  
We call  \verb@npGSEASummary@ to get a summary of the results. 
\verb@T_Gw@ is explained in more detail
in Section 3.2.   

<<runIt>>=
yFactor <- ALL$mol.biol
res1 <- npGSEA(x = ALL, y = yFactor, set = set1)  ##with the eset
res1

res2_exprs <- npGSEA(xData, ALL$mol.biol, gsc[[2]])  ##with the expression data
res2_exprs 
@

\verb@npGSEA@ has several built in accessor functions to gather more 
information about the analysis of your set of interest 
in your experiment.  
<<accessorFunctions>>=
res3 <- npGSEA(ALL, yFactor, set3)  
res3
geneSetName(res3)
stat(res3)
sigmaSq(res3)
zStat(res3)
pTwoSided(res3)
pLeft(res3)
pValues(res3)
dim(xSet(res3))
@

There is also a \verb@npGSEA@ specific plot function (\verb@npGSEAPlot@) to 
visualize the results of your analysis.  Highlighted in red on the plot is 
the corresponding \verb@zStat@ of our analysis.
<<plot3, fig = TRUE>>=
npGSEAPlot(res3)
@

\begin{figure}
\centering
\includegraphics{npGSEA-plot3}
\caption{
  \textbf{Set3 normal approximation results}  
  This plot displays the standard normal curve and our observed
  zStat for set3 in this analysis.
 }
\end{figure}


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\subsection{Running npGSEA with the beta and chi-sq approximations}
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
There are three types of approximation methods in \verb@npGSEA@: 
"norm", "beta", and "chiSq".
Each method is discussed in brief in Section 3.  The "norm" 
approximation method is the default. Note that each of these 
methods has the same $\hat\beta_{g}$ (see methods section).

<<runItNorm, fig = TRUE>>=
res5_norm <- npGSEA(ALL, yFactor, set5, approx= "norm")  
res5_norm
betaHats(res5_norm)
npGSEAPlot(res5_norm)
@



The beta approximation yields results quite similar to the normal approximation.  
<<runItBeta, fig = TRUE>>=
res5_beta <- npGSEA(ALL, yFactor, set5, approx= "beta")
res5_beta
betaHats(res5_beta)
npGSEAPlot(res5_beta)
@

The chi-sq approximation method is only available for the two-sided test.  
Here we call \verb@npGSEA@ and 
then show how the \verb@chiSqStat@ is related to \verb@C_Gw@.  
\verb@C_Gw@ is explained in more detail
in Section 3.2.   

<<runItChiSq, fig = TRUE>>=
res5_chiSq <- npGSEA(ALL, yFactor, set5, approx= "chiSq")  
res5_chiSq
betaHats(res5_chiSq)
chiSqStat(res5_chiSq)
stat(res5_chiSq)
stat(res5_chiSq)/sigmaSq(res5_chiSq)
npGSEAPlot(res5_chiSq)
@

Note that, as we expected, \verb@set5@ is a significantly enriched 
in all three methods.  In each of the three 
corresponding plots, the observed statistic is a very rare event.

\begin{figure}
\centering
\includegraphics[width=.3\textwidth]{npGSEA-runItNorm}
\includegraphics[width=.3\textwidth]{npGSEA-runItBeta}
\includegraphics[width=.3\textwidth]{npGSEA-runItChiSq}
\caption{
  \textbf{Set5 normal, beta, and chi-sq approximation results}  
  These plots displays the reference normal, beta, and chi-sq curves, 
  and our observed
 zStat, betaStat, and chiSqStat for set5 in this analysis.
 }
\end{figure}


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\subsection{Adding weights to the model}
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Sometimes we do not want to weigh each gene in our set equally.  
We want to assign a larger 
weight to genes that are of 
a particular interest, and a lower weight to genes that we know may 
behave poorly.  In this example, we
weight the genes in \verb@set7@ by their variance.

<<runItwts>>=
res7_nowts <- npGSEA(x = ALL, y= yFactor, set = set7)  
res7_nowts

wts <- apply(exprs(ALL)[match(geneIds(set7), featureNames(ALL)), ], 
						1, var)	
wts <- 1/wts				
res7_wts <- npGSEA(x = ALL, y = yFactor, set = set7,  w = wts, approx= "norm")  
res7_wts						
@



By adding these weights, we get a slightly more significant result.  
We can add weights for the beta and 
chi-sq approximations, too.  By default, \verb@npGSEA@ assigns 
a weight of 1 for all genes.


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\subsection{Adding covariates to model}
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Often we want to correct for confounders in our model.  To do this with 
\verb@npGSEA@, we provide a vector or matrix in the \verb@covars@ 
slot of our function.  \verb@npGSEA@ then projects both the data (\verb@x@) 
and the outcome of interest (\verb@y@) against our covariate matrix/vector.  
The resulting residuals are used for further analysis.

In this 
example, we correct for the age and sex of the subjects in our experiment. 
For more details on model selection and its relation to inference, 
please see the \verb@limma@ manual.  

<<addCovariates>>=
res3_age <- npGSEA(x = ALL, y = yFactor, set = set3, covars = ALL$age) 
res3_age

res3_agesex <- npGSEA(x = ALL, y = yFactor, set = set3, covars = cbind(ALL$age, ALL$sex)) 
res3_agesex
@

By adjusting for these variables, we get a slight different result than above.  Note that we can adjust for 
covariates in the beta and chi-sq approximation methods, too.


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\subsection{Running npGSEA with multiple gene sets}
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
To explore multiple gene sets, we let \verb@set@ be a 
\verb@GeneSetCollection@.  This returns a list of 
\verb@npGSEAResultNorm@ objects, called a 
\verb@npGSEAResultNormCollection@.  We can access 
statistics for each \verb@GeneSet@ in our analysis through 
accessors of \verb@npGSEAResultNormCollection@.  

<<runMultiplenpGSEA>>=
resgsc_norm <- npGSEA(x = ALL, y = yFactor, set = gsc)  
unlist( pLeft(resgsc_norm) )
unlist( stat (resgsc_norm) )
unlist( zStat (resgsc_norm) )
@

Note how quick our method is.  We get results as accurate as 
permutation methods in a fraction of the time, even for multiple gene sets.  

Using the \verb@ReportingTools@ package, we can publish 
these results to a HTML page for exploration.  We first adjust for multiple testing.
<<publishMultiplenpGSEA, eval=FALSE >>=
pvals <- p.adjust( unlist(pTwoSided(resgsc_norm)), method= "BH" )
library(ReportingTools)
npgseaReport <- HTMLReport (shortName = "npGSEA",
        title = "npGSEA Results", reportDirectory = "./reports")
publish(gsc, npgseaReport, annotation.db = "org.Hs.eg",
        setStats = unlist(zStat (resgsc_norm)), setPValues = pvals)
finish(npgseaReport)
@



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\section{Methods in brief}
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\subsection{Disadvantages to a permutation approach}
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
There are three main disadvantages to permutation-based analyses: cost,
randomness, and granularity.

Testing many sets of genes 
becomes computationally expensive for two reasons.  
First, there are many test statistics to calculate
in each permuted version of the data. 
Second, to allow for multiplicity adjustment, we
require small nominal $p$-values to draw inference about our sets, 
which in turn requires a large number
of permutations.

Permutations are also subject random inference.  
Because permutations are based on a random shuffling of the data,
there is a chance that we will obtain a different $p$-value 
for our set of interest each 
time we run our permutation analysis.  

Permutations also have a granularity problem.
If we do $M$ permutations, then the smallest possible
$p$-value we can attain is $1/(M+1)$.  When it is necessary
to adjust for multiplicity, the permutation approach becomes 
very computationally expensive.  
Another aspect of the granularity problem is that permutations
give us no basis to distinguish between two gene sets that
both have the same $p$-value $1/(M+1)$. There may be many such gene sets,
and they have meaningfully different effect sizes.  

Because of each of these limitations of permutation testing, there is a 
need to move beyond
permutation-based GSEA methods.  The methods we present in 
\verb@npGSEA@ and discuss 
in brief below are not as computationally 
expensive, random, or granular than their permutation counterparts.  
More details on our method can 
be found in Larson and Owen (2015).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\subsection{Test statistics}
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
We present our notation using the language of gene expression
experiments.
  
Let $g$ and $h$ denote individual genes
and $G$ be a set of genes.  Our experiment has $n$ subjects.
The subjects may represent patients, cell cultures, or tissue samples.  
The expression level for gene $g$ in subject $i$ 
is $X_{gi}$, and $Y_{i}$ is the target variable on subject $i$. 
$Y_{i}$ is often a treatment, disease, or genotype.  We center the variables so that
$\sum_{i=1}^nY_{i} = \sum_{i=1}^n X_{gi} = 0, \forall g$.

Our measure of association for gene $g$ on our treatment of interest is
$$\hat\beta_{g} = \frac1{n}\sum_{i=1}^nX_{gi}Y_{i}.$$

We consider the linear statistic
$$ T_{G,w}  = \sum_{g\in G}w_{g}\hat\beta_{g}$$
and the quadratic statistic 
$$C_{G,w} = \sum_{g\in G}w_{g}\hat\beta_{g}^2,$$ 
where $w_{g}$
corresponds to the weight given to gene $g$ in set $G$. 




%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\subsection{Moment based reference distributions}
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
To avoid the issues discussed above, we approximate the distribution of
the permuted test statistics $T_{G,w}$ by Gaussian
or by rescaled beta distributions.
For the quadratic statistic $C_{G,w}$ we use
a distribution of the form $\sigma^2\chi^2_{(\nu)}$.

For the Gaussian treatment of $T_{G,w}$ we
calculate $\sigma^2 = Var(T_{G,w})$ under permutation,
and then report the $p$-value
$$
p = Pr( N( 0, \sigma^2 ) \le T_{G,w}).
$$
The above is a left tail $p$-value. Two-sided and right tailed $p$-values are 
analogous.  

When we want something sharper than the normal distribution, we can
use a scaled Beta distribution, of the form 
$A + (B-A)Beta(\alpha,\beta)$. 
The $Beta(\alpha,\beta)$ distribution has 
a continuous density function
on $0<x<1$ for $\alpha,\beta>0$.
We choose $A$, $B$, $\alpha$ and $\beta$ by matching the upper
and lower limits of $T_{G,w}$ under permutation, as well as its mean and variance.  
The observed left tailed $p$-value is
$$
p = Pr\Bigl( Beta(\alpha,\beta) \le \frac{T_{G,w}-A}{B-A}\Bigr).
$$

For the quadratic test statistic $C_{G,w}$
we use a $\sigma^2\chi^2_{(\nu)}$ reference distribution
reporting the $p$-value
$$\Pr( \sigma^2\chi^2_{(\nu)}\ge C_{G,w}),$$
after matching the first and second moments of $\sigma^2\chi^2_{(\nu)}$
to $E(C_{G,w})$ and $E( C_{G,w}^2)$ under permutation, respectively.


Additional details on how $\sigma^2$, $A$, $B$, $\alpha$, $\beta$, $E(C_{G,w})$, $E( C_{G,w}^2)$, and 
$\nu$ 
are derived can be found in Larson and Owen (2015).



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\section{Session Info}
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
<<sessInfo, results=tex, echo=FALSE>>=
toLatex(sessionInfo())
@

<<resetOptions, echo=FALSE>>=
options(prompt="> ", continue="+ ")
@ 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\section{References}
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Larson and Owen.  (2015).  Moment based gene set tests.  \emph{BMC Bioinformatics.}  \textbf{16}:132. \url{http://www.biomedcentral.com/1471-2105/16/132}

\end{document}

\name{npGSEAResultBeta-class}
\Rdversion{1.1}
\docType{class}
\alias{npGSEAResultBeta-class}

\title{Class \code{"npGSEAResultBeta"}}
\description{
Objects of this class store results from running npGSEA with the beta approximation. 
}
\section{Objects from the Class}{
Objects can be created by calls of \code{npGSEA}.
}
\section{Slots}{
    \describe{
        \item{\code{geneSetName}:}{Object of class \code{"character"}, the name of the geneSet }
        \item{\code{betaStat}:}{Object of class \code{"numeric"} , the test statistic, scaled to the corresponding beta distribution }
        \item{\code{ThatGw}:}{Object of class \code{"numeric"}, the test statistic for the set }
        \item{\code{varThatGw}:}{Object of class \code{"numeric"}, the variance of ThatGw }
        \item{\code{alpha}:}{Object of class \code{"numeric"}, the alpha value }
        \item{\code{beta}:}{Object of class \code{"numeric"}, the beta value}
        \item{\code{pLeft}:}{Object of class \code{"numeric"}, the p-value for the left-side hypothesis }
        \item{\code{pRight}:}{Object of class \code{"numeric"}, the p-value for the right-side hypothesis }
        \item{\code{pTwoSided}:}{Object of class \code{"numeric"}, the p-value for the two-sided hypothesis }
        \item{\code{xSet}:}{Object of class \code{"matrix"}, the centered and scaled x data for this set}
        \item{\code{betaHats}:}{Object of class \code{"vector"}, the betaHats for each gene in this set}
    }
}
\author{
Jessica L. Larson
}

\seealso{
\code{\linkS4class{npGSEAResultNorm}}
}
\examples{
showClass("npGSEAResultBeta")
}
\keyword{classes}
\references{
Jessica L Larson and Art B Owen: Moment based gene set tests. BMC Bioinformatics 2015, 16:132. \url{http://www.biomedcentral.com/1471-2105/16/132}
}

\name{sigmaSq-methods}
\docType{methods}
\alias{sigmaSq}
\alias{sigmaSq-methods}
\alias{sigmaSq,npGSEAResultNorm-method}
\alias{sigmaSq,npGSEAResultBeta-method}
\alias{sigmaSq,npGSEAResultChiSq-method}
\alias{sigmaSq,npGSEAResultNormCollection-method}
\alias{sigmaSq,npGSEAResultBetaCollection-method}
\alias{sigmaSq,npGSEAResultChiSqCollection-method}
\title{ ~~ Methods for Function \code{sigmaSq}  ~~}
\description{
This function returns the corresponding variance of the statistic (linear or quadratic) from the npGSEA analysis 
 for a given \code{GeneSet},
or a list of these variances for a given \code{GeneSetCollection}.  
This method is applicable for all three approximation methods.
}
\usage{sigmaSq(object)}
\arguments{
  \item{object}{An object of type \code{npGSEAResultNorm}, \code{npGSEAResultBeta}, 
      \code{npGSEAResultChiSq},
      \code{npGSEAResultNormCollection}, \code{npGSEAResultBetaCollection}, or
       \code{npGSEAResultChiSqCollection} }
}

\section{Methods}{
\describe{
    \item{\code{signature(object = "npGSEAResultNorm")}}{Returns the variance 
    of the linear statistic for a \code{npGSEAResultNorm} object}  
    \item{\code{signature(object = "npGSEAResultBeta")}}{Returns the variance 
    of the linear statistic for a \code{npGSEAResultBeta} object} 
    \item{\code{signature(object = "npGSEAResultChiSq")}}{Returns the variance 
    of the quadratic statistic for a \code{npGSEAResultChiSq} object} 
    \item{\code{signature(object = "npGSEAResultNormCollection")}}{Returns a list 
    of the variances of the linear statistics for a \code{npGSEAResultNormCollection} 
    objects (1 for each set)}
    \item{\code{signature(object = "npGSEAResultBetaCollection")}}{Returns a list of 
    the variances of the linear statistics for a \code{npGSEAResultBetaCollection} 
    objects (1 for each set)}
    \item{\code{signature(object = "npGSEAResultChiSqCollection")}}{Returns a list of 
    the variances of the linear statistics for a \code{npGSEAResultChiSqCollection} 
    objects (1 for each set)} 
    }
}
\seealso{
  \code{\linkS4class{npGSEAResultNorm}}-class
}
\examples{
    set.seed(15)
    yFactor <- as.factor( c(rep("treated", 5), rep("control", 5)) )
    xData <- matrix(data=rnorm(length(letters)*10) ,nrow=length(letters), ncol=10)
    rownames(xData) <- letters
    geneSetABC15 <- GeneSet(geneIds=letters[1:15], setName="setABC15")
    res <- npGSEA(x = xData, y = yFactor, set = geneSetABC15)
    sigmaSq(res)
}
\author{
Jessica L. Larson
}
\keyword{methods}

\name{xSet-methods}
\docType{methods}
\alias{xSet}
\alias{xSet-methods}
\alias{xSet,npGSEAResultNorm-method}
\alias{xSet,npGSEAResultBeta-method}
\alias{xSet,npGSEAResultChiSq-method}
\alias{xSet,npGSEAResultNormCollection-method}
\alias{xSet,npGSEAResultBetaCollection-method}
\alias{xSet,npGSEAResultChiSqCollection-method}
\title{ ~~ Methods for Function \code{xSet}  ~~}
\description{
This function returns the scaled and centered expression data 
for all genes in the corresponding \code{GeneSet} in the given experiment, 
or a list of such matrices for each set in a \code{GeneSetCollection}.  
This method is applicable for all three approximation methods.
}

\usage{
    xSet(object)
}
\arguments{
    \item{object}{An object of type \code{npGSEAResultNorm}, \code{npGSEAResultBeta}, 
    \code{npGSEAResultChiSq},\code{npGSEAResultNormCollection}, 
    \code{npGSEAResultBetaCollection}, or \code{npGSEAResultChiSqCollection} }
}

\section{Methods}{
\describe{
    \item{\code{signature(object = "npGSEAResultNorm")}}{Returns the centered and 
    scaled X matrix used for analysis to create a \code{npGSEAResultNorm} object}  
    \item{\code{signature(object = "npGSEAResultBeta")}}{Returns the centered and 
    scaled X matrix used for analysis to create a \code{npGSEAResultBeta} object}    
    \item{\code{signature(object = "npGSEAResultChiSq")}}{Returns the centered and 
    scaled X matrix used for analysis to create a \code{npGSEAResultChiSq} object}
    \item{\code{signature(object = "npGSEAResultNormCollection")}}{Returns a list of 
    centered and scaled X matrices used for analysis to create the 
    \code{npGSEAResultNormCollection} objects (1 for each set)}    
    \item{\code{signature(object = "npGSEAResultBetaCollection")}}{Returns a list of 
    centered and scaled X matrices used for analysis to create the 
    \code{npGSEAResultBetaCollection} objects (1 for each set)}    
    \item{\code{signature(object = "npGSEAResultChiSqCollection")}}{Returns a list of 
    centered and scaled X matrices used for analysis to create the 
    \code{npGSEAResultChiSqCollection} objects (1 for each set)}
    }
}
\seealso{
  \code{\linkS4class{npGSEAResultNorm}}-class
}

\examples{
    set.seed(15)
    yFactor <- as.factor( c(rep("treated", 5), rep("control", 5)) )
    xData <- matrix(data=rnorm(length(letters)*10) ,nrow=length(letters), ncol=10)
    rownames(xData) <- letters
    geneSetABC15 <- GeneSet(geneIds=letters[1:15], setName="setABC15")
    res <- npGSEA(x = xData, y = yFactor, set = geneSetABC15)
    head( xSet(res) )
}


\keyword{methods}

\name{npGSEAResultNormCollection-class}
\Rdversion{1.1}
\docType{class}
\alias{npGSEAResultNormCollection-class}

\title{Class \code{"npGSEAResultNormCollection"}}
\description{
Objects of this class store results from running npGSEA with the Gaussian approximation with a GeneSetCollection.   
npGSEAResultNormCollection objects contain a list of npGSEAResultNorm objects (one result for each GeneSet).
}
\section{Objects from the Class}{
Objects can be created by calls of \code{npGSEA}.
}
\author{
Jessica L. Larson
}

\seealso{
\code{\linkS4class{npGSEAResultChiSqCollection}}
}
\examples{
showClass("npGSEAResultNormCollection")
}
\keyword{classes}
\references{
Jessica L Larson and Art B Owen: Moment based gene set tests. BMC Bioinformatics 2015, 16:132. \url{http://www.biomedcentral.com/1471-2105/16/132}
}

\name{npGSEA}
\alias{npGSEA}
\title{Calculates an approximation of the permutation GSEA statistics and p-values}
\usage{
  npGSEA(x, y, set, covars = NULL, approx = c("norm", "beta", "chiSq"), w = NULL, epsilonBetaAdj=TRUE, scaleXY=TRUE, uniVarX=TRUE )
}
\arguments{
  \item{x}{A matrix of expression data or an object of type ExpressionSet.  
      The columns of x represent samples in a given experiment.  The rows are genes.  
      The names of each row (or \code{featureNames} of the eSet) 
      must be of the same type (e.g., entrez ids) as the ids of the gene set.}

  \item{y}{A vector containing the treatment for each sample. 
      The length of y must be more than 4 for the "chisq" approximation.  
      Each treatment group must have at least two observations for all approximation methods.  
      There can only be two treatment groups.}
  
  \item{covars}{A vector or matrix containing covariate(s) of interest, optional}
  
  \item{set}{A \code{GeneSet} object containing a set of genes of interest or a 
  	\code{GeneSetCollection} object containing a collection of GeneSets}

  \item{approx}{A string of either "norm" (default), "beta" or "chiSq".  
      If "norm", the normal approximation to the non-permutation GSEA is calculated and returned.  
      If "beta", the beta approximation is reported.  If "chiSq", the Chi-squared approximation to the 
      permutation GSEA is calculated.}

  \item{w}{A vector or list containing the weights of each gene in the set or sets, optional.  If w is a list, 
  	the number of elements in the list must correspond to the number of gene sets in the collection.}
  	
  	  \item{epsilonBetaAdj}{A boolean indicating whether or to not to use an epsilon adjusted p-value for the Beta approximation.  
  	  	When TRUE, this prevents observed p-values of 0.  The default is TRUE.}
  	  	
  	   \item{scaleXY}{A boolean indicating whether or to not to scale x and y.  The default is TRUE.}
  	   \item{uniVarX}{A boolean indicating whether or to not to scale x to have unit variance. The default is TRUE.}
}
\value{
An object with the corresponding GSEA results.  If approx="norm" an \code{npGSEAResultNorm} 
object is returned.  If approx="beta" a \code{npGSEAResultBeta} object is returned.  If approx="chiSq" 
a \code{npGSEAResultChiSq object} is returned.  If set is a \code{GeneSetCollection} (i.e., multiple 
sets of interest), then the corresponding \code{npGSEAResultNormCollection}, 
\code{npGSEAResultBetaCollection}, or \code{npGSEAResultChiSqCollection} is returned.
  }
\description{
  This function calculates the permutation gene set enrichment analysis test statistic and p-value 
  without actually running the permutation.  We account for the covariance among the genes within 
  the set and approximate the corresponding permutation distribution.  For more details on the 
  method see Larson and Owen (2014).
}
\author{
  Jessica L. Larson and Art Owen
}
\examples{
    set.seed(15)
    yFactor <- as.factor( c(rep("treated", 5), rep("control", 5)) )
    xData <- matrix(data=rnorm(length(letters)*10) ,nrow=length(letters), ncol=10)
    rownames(xData) <- letters
    geneSetABC15 <- GeneSet(geneIds=letters[1:15], setName="setABC15")
    res <- npGSEA(x = xData, y = yFactor, set = geneSetABC15)
}
\references{
Jessica L Larson and Art B Owen: Moment based gene set tests. BMC Bioinformatics 2015, 16:132. \url{http://www.biomedcentral.com/1471-2105/16/132}
}

\name{npGSEAPlot-methods}
\docType{methods}
\alias{npGSEAPlot}
\alias{npGSEAPlot-methods}
\alias{npGSEAPlot,npGSEAResultNorm-method}
\alias{npGSEAPlot,npGSEAResultBeta-method}
\alias{npGSEAPlot,npGSEAResultChiSq-method}
\title{ ~~ Methods for Function \code{npGSEAPlot}  ~~}
\description{
This function plots the reference distribution and the 
corresponding scaled statistic (Z, Beta, or Chi-sq) from the npGSEA analysis 
 for a given \code{GeneSet}.  
This method is applicable for all three approximation methods.
}
\usage{
    npGSEAPlot(object)
}
\arguments{
    \item{object}{ An object of type \code{npGSEAResultNorm}, \code{npGSEAResultBeta}, 
      or \code{npGSEAResultChiSq} }
}
\section{Methods}{
\describe{
    \item{\code{signature(object = "npGSEAResultNorm")}}{Plots the Z-statistic for a 
    \code{npGSEAResultNorm} object and the standard normal distribution}
    \item{\code{signature(object = "npGSEAResultBeta")}}{Plots the beta statistic 
    for a \code{npGSEAResultBeta} object and the corresponding reference beta 
    distribution (with alpha and beta calculated from \code{npGSEA}).}   
    \item{\code{signature(object = "npGSEAResultChiSq")}}{Plots the beta 
    statistic for a \code{npGSEAResultChiSq} object and the corresponding reference 
    chi-squared distribution (with degrees of freedom calculated from \code{npGSEA}).}
}
}
\seealso{\code{\linkS4class{npGSEAResultNorm}}-class}
\examples{
    set.seed(15)
    yFactor <- as.factor( c(rep("treated", 5), rep("control", 5)) )
    xData <- matrix(data=rnorm(length(letters)*10) ,nrow=length(letters), ncol=10)
    rownames(xData) <- letters
    geneSetABC15 <- GeneSet(geneIds=letters[1:15], setName="setABC15")
    res <- npGSEA(x = xData, y = yFactor, set = geneSetABC15)
    ##npGSEAPlot (res)
}
\author{
Jessica L. Larson
}
\keyword{methods}

\name{pTwoSided-methods}
\docType{methods}
\alias{pTwoSided}
\alias{pTwoSided-methods}
\alias{pTwoSided,npGSEAResultNorm-method}
\alias{pTwoSided,npGSEAResultBeta-method}
\alias{pTwoSided,npGSEAResultChiSq-method}
\alias{pTwoSided,npGSEAResultNormCollection-method}
\alias{pTwoSided,npGSEAResultBetaCollection-method}
\alias{pTwoSided,npGSEAResultChiSqCollection-method}
\title{ ~~ Methods for Function \code{pTwoSided}  ~~}
\description{
This function returns the two-sided p-value for the corresponding \code{GeneSet} 
or a list of p-values for a \code{GeneSetCollection}.  
This method is applicable for all three approximation methods.
}
\usage{pTwoSided(object)}
\arguments{
  \item{object}{An object of type \code{npGSEAResultNorm}, \code{npGSEAResultBeta}, 
    \code{npGSEAResultChiSq}, \code{npGSEAResultNormCollection}, 
    \code{npGSEAResultBetaCollection}, or \code{npGSEAResultChiSqCollection} }
}
\section{Methods}{
\describe{
    \item{\code{signature(object = "npGSEAResultNorm")}}{Returns a two-sided 
    p-value for a \code{npGSEAResultNorm} object}
    \item{\code{signature(object = "npGSEAResultBeta")}}{Returns a two-sided 
    p-value for a \code{npGSEAResultBeta} object}    
    \item{\code{signature(object = "npGSEAResultChiSq")}}{Returns a two-sided 
    p-value for a \code{npGSEAResultChiSq} object}
    \item{\code{signature(object = "npGSEAResultNormCollection")}}{Returns a list 
    of left-sided p-values for a \code{npGSEAResultNormCollection} objects 
    (1 for each set)}
    \item{\code{signature(object = "npGSEAResultBetaCollection")}}{Returns a list of 
    left-sided p-values for a \code{npGSEAResultBetaCollection} objects (1 for each set)} 
    \item{\code{signature(object = "npGSEAResultChiSqCollection")}}{Returns a list of 
    two-sided p-values for a \code{npGSEAResultChiSqCollection} objects (1 for each set)}
    }
}

\seealso{
    \code{\linkS4class{npGSEAResultNorm}}-class, \code{pRight}
}

\examples{
    set.seed(15)
    yFactor <- as.factor( c(rep("treated", 5), rep("control", 5)) )
    xData <- matrix(data=rnorm(length(letters)*10) ,nrow=length(letters), ncol=10)
    rownames(xData) <- letters
    geneSetABC15 <- GeneSet(geneIds=letters[1:15], setName="setABC15")
    res <- npGSEA(x = xData, y = yFactor, set = geneSetABC15)
    pTwoSided (res)
}
\author{
Jessica L. Larson
}
\keyword{methods}

\name{summary-methods}
\docType{methods}
\alias{show}
\alias{show-methods}
\alias{show,npGSEAResultNorm-method}
\alias{show,npGSEAResultBeta-method}
\alias{show,npGSEAResultChiSq-method}
\title{ ~~ Methods for Function \code{show} in Package \pkg{base} ~~}
\description{
 ~~ Methods for function \code{show} in package \pkg{base} ~~
}
\section{Methods}{
\describe{
    \item{
        \code{signature(x = "npGSEAResultNorm")},\code{signature(x = "npGSEAResultBeta")},
        \code{signature(x = "npGSEAResultChiSq")}}{These methods display the corresponding 
        statistics (linear or quadratic)for the npGSEA analysis in the gene set in the given experiment.}
    }
}
\examples{
    set.seed(15)
    yFactor <- as.factor( c(rep("treated", 5), rep("control", 5)) )
    xData <- matrix(data=rnorm(length(letters)*10) ,nrow=length(letters), ncol=10)
    rownames(xData) <- letters
    geneSetABC15 <- GeneSet(geneIds=letters[1:15], setName="setABC15")
    res <- npGSEA(x = xData, y = yFactor, set = geneSetABC15)
    res
}
\author{Jessica L. Larson}
\keyword{methods}

\name{betaValue-methods}
\docType{methods}
\alias{betaValue}
\alias{betaValue-methods}
\alias{betaValue,npGSEAResultBeta-method}
\alias{betaValue,npGSEAResultBetaCollection-method}
\title{ ~~ Methods for Function \code{betaValue}  ~~}
\description{
This function returns the corresponding beta value for the reference 
beta distribution for the \code{npGSEA} analysis in the gene set in the 
given experiment.  This method is applicable for only the beta approximation method.
}
\usage{betaValue(object)}
\arguments{
    \item{object}{An object of type 
        \code{npGSEAResultBeta} or \code{npGSEAResultBetaCollection}}
}
\section{Methods}{
\describe{
        \item{\code{signature(object = "npGSEAResultBeta")} }{Returns the 
        value for beta for a \code{npGSEAResultBeta} object}
        \item{\code{signature(object = "npGSEAResultBetaCollection")} }{Returns a 
        list of the beta values for a \code{npGSEAResultBetaCollection} objects (1 for each set)}
        }
}
\seealso{
    \code{\linkS4class{npGSEAResultBeta}}-class
}
\examples{
    set.seed(15)
    yFactor <- as.factor( c(rep("treated", 5), rep("control", 5)) )
    xData <- matrix(data=rnorm(length(letters)*10) ,nrow=length(letters), ncol=10)
    rownames(xData) <- letters
    geneSetABC15 <- GeneSet(geneIds=letters[1:15], setName="setABC15")
    res <- npGSEA(x = xData, y = yFactor, set = geneSetABC15, approx= "beta")  
    betaValue(res)
}
\author{Jessica L. Larson}
\keyword{methods}

\name{betaHats-methods}
\docType{methods}
\alias{betaHats}
\alias{betaHats-methods}
\alias{betaHats,npGSEAResultNorm-method}
\alias{betaHats,npGSEAResultBeta-method}
\alias{betaHats,npGSEAResultChiSq-method}
\alias{betaHats,npGSEAResultNormCollection-method}
\alias{betaHats,npGSEAResultBetaCollection-method}
\alias{betaHats,npGSEAResultChiSqCollection-method}
\title{ ~~ Methods for Function \code{betaHats}  ~~}
\description{
This function returns the betaHats 
for all genes in the corresponding \code{GeneSet} in the given experiment, 
or a list of such vectors for each set in a \code{GeneSetCollection}.  
This corresponds to each gene's contrubution to the test statistic.
This method is applicable for all three approximation methods.
}

\usage{
    betaHats(object)
}
\arguments{
    \item{object}{An object of type \code{npGSEAResultNorm}, \code{npGSEAResultBeta}, 
    \code{npGSEAResultChiSq},\code{npGSEAResultNormCollection}, 
    \code{npGSEAResultBetaCollection}, or \code{npGSEAResultChiSqCollection} }
}

\section{Methods}{
\describe{
    \item{\code{signature(object = "npGSEAResultNorm")}}{Returns the betaHats 
    	used for analysis to create a \code{npGSEAResultNorm} object}  
    \item{\code{signature(object = "npGSEAResultBeta")}}{Returns the betaHats 
    used for analysis to create a \code{npGSEAResultBeta} object}    
    \item{\code{signature(object = "npGSEAResultChiSq")}}{Returns the betaHats
    used for analysis to create a \code{npGSEAResultChiSq} object}
    \item{\code{signature(object = "npGSEAResultNormCollection")}}{Returns the betaHats 
    	used for analysis to create the 
    \code{npGSEAResultNormCollection} objects (1 for each set)}    
    \item{\code{signature(object = "npGSEAResultBetaCollection")}}{Returns a list of 
    the betaHats used for analysis to create the 
    \code{npGSEAResultBetaCollection} objects (1 for each set)}    
    \item{\code{signature(object = "npGSEAResultChiSqCollection")}}{Returns a list of 
    the betaHats used for analysis to create the 
    \code{npGSEAResultChiSqCollection} objects (1 for each set)}
    }
}
\seealso{
  \code{\linkS4class{npGSEAResultNorm}}-class
}

\examples{
    set.seed(15)
    yFactor <- as.factor( c(rep("treated", 5), rep("control", 5)) )
    xData <- matrix(data=rnorm(length(letters)*10) ,nrow=length(letters), ncol=10)
    rownames(xData) <- letters
    geneSetABC15 <- GeneSet(geneIds=letters[1:15], setName="setABC15")
    res <- npGSEA(x = xData, y = yFactor, set = geneSetABC15)
    betaHats(res) 
}


\keyword{methods}

\name{getIncidence}
\alias{getIncidence}
\title{Determines the incidence of a gene set in a list of genes.}
\usage{
  getIncidence(universeIDs, set)
}
\arguments{
    \item{universeIDs}{A vector containing the list of 
        possible gene ids in the universe (or experiment).}

    \item{set}{A GeneSet object containing a set of genes of
        interest}
}
\value{
    A list of inSet and inExp.  inSet is a vector with the same length as universeIDs.  
    Each value of inSet is 1 if the gene is in the set and 0 otherwise.  
    inExp is a vector with the same length as geneIds(set), the number of genes in the set.  
    Each value of inExp is 1 if the gene is in universeIDs and 0 otherwise.
}
\description{
    Calculates the incidence of a gene set in an experiment
}
\details{
    getIncidence returns an incidence vector of the location of the genes within a gene set in a list of genes in an experiment and vise-versa.
}
\examples{
    geneSetABC15 <- GeneSet(geneIds=letters[1:15], setName="setABC15")
    getIncidence(letters, geneSetABC15)
}
\author{
    Jessica L. Larson and Art Owen
}
\references{
Jessica L Larson and Art B Owen: Moment based gene set tests. BMC Bioinformatics 2015, 16:132. \url{http://www.biomedcentral.com/1471-2105/16/132}
}

\name{pRight-methods}
\docType{methods}
\alias{pRight}
\alias{pRight-methods}
\alias{pRight,npGSEAResultNorm-method}
\alias{pRight,npGSEAResultBeta-method}
\alias{pRight,npGSEAResultNormCollection-method}
\alias{pRight,npGSEAResultBetaCollection-method}
\title{ ~~ Methods for Function \code{pRight}  ~~}
\description{
This function returns the right-sided p-value for the corresponding \code{GeneSet} 
or a list of p-values for a \code{GeneSetCollection}.  
This method is only applicable for the normal and beta approximation methods.
}
\usage{pRight(object)}
\arguments{
\item{object}{An object of type \code{npGSEAResultNorm}, \code{npGSEAResultBeta}, 
\code{npGSEAResultNormCollection}, or \code{npGSEAResultBetaCollection} }
}
\section{Methods}{
\describe{
    \item{\code{signature(object = "npGSEAResultNorm")}}{Returns a right-sided 
    p-value for a \code{npGSEAResultNorm} object}
    \item{\code{signature(object = "npGSEAResultBeta")}}{Returns a right-sided 
    p-value for a \code{npGSEAResultBeta} object}
    \item{\code{signature(object = "npGSEAResultNormCollection")}}{Returns a list 
    of right-sided p-values for a \code{npGSEAResultNormCollection} objects 
    (1 for each set)}
    \item{\code{signature(object = "npGSEAResultBetaCollection")}}{Returns a list of 
    right-sided p-values for a \code{npGSEAResultBetaCollection} objects 
    (1 for each set)}
}
}
\seealso{
\code{\linkS4class{npGSEAResultNorm}}-class, \code{pLeft}
}
\examples{
    set.seed(15)
    yFactor <- as.factor( c(rep("treated", 5), rep("control", 5)) )
    xData <- matrix(data=rnorm(length(letters)*10) ,nrow=length(letters), ncol=10)
    rownames(xData) <- letters
    geneSetABC15 <- GeneSet(geneIds=letters[1:15], setName="setABC15")
    res <- npGSEA(x = xData, y = yFactor, set = geneSetABC15)
    pRight (res)
}
\author{
Jessica L. Larson
}
\keyword{methods}

\name{alphaValue-methods}
\docType{methods}
\alias{alphaValue}
\alias{alphaValue-methods}
\alias{alphaValue,npGSEAResultBeta-method}
\alias{alphaValue,npGSEAResultBetaCollection-method}
\title{ ~~ Methods for Function \code{alphaValue}  ~~}
\description{
This function returns the corresponding alpha value for the reference 
beta distribution for the \code{npGSEA} analysis in the gene set in the 
given experiment.  This method is applicable for only the beta approximation method.
}
\usage{alphaValue(object)}
\arguments{
    \item{object}{An object of type 
        \code{npGSEAResultBeta} or \code{npGSEAResultBetaCollection}}
}
\section{Methods}{
\describe{
    \item{\code{signature(object = "npGSEAResultBeta")}}{Returns the value for 
    	alpha for a \code{npGSEAResultBeta} object}
    \item{\code{signature(object = "npGSEAResultBetaCollection")}}{Returns a 
    	list of the alpha values for a \code{npGSEAResultBetaCollection} 
        objects (1 for each set)}}
}
\seealso{
    \code{\linkS4class{npGSEAResultBeta}}-class
}
\examples{
    set.seed(15)
    yFactor <- as.factor( c(rep("treated", 5), rep("control", 5)) )
    xData <- matrix(data=rnorm(length(letters)*10) ,nrow=length(letters), ncol=10)
    rownames(xData) <- letters
    geneSetABC15 <- GeneSet(geneIds=letters[1:15], setName="setABC15")
    res <- npGSEA(x = xData, y = yFactor, set = geneSetABC15, approx= "beta")  
    alphaValue(res)
}
\author{Jessica L. Larson}
\keyword{methods}

\name{geneSetName-methods}
\docType{methods}
\alias{geneSetName}
\alias{geneSetName-methods}
\alias{geneSetName,npGSEAResultNorm-method}
\alias{geneSetName,npGSEAResultBeta-method}
\alias{geneSetName,npGSEAResultChiSq-method}
\alias{geneSetName,npGSEAResultNormCollection-method}
\alias{geneSetName,npGSEAResultBetaCollection-method}
\alias{geneSetName,npGSEAResultChiSqCollection-method}
\title{ ~~ Methods for Function \code{geneSetName}  ~~}
\description{
This function returns the name of the corresponding \code{GeneSet} 
or a list of names for a \code{GeneSetCollection}.  
This method is applicable for all three approximation methods.
}
\usage{geneSetName(object)}
\arguments{
  \item{object}{An object of type \code{npGSEAResultNorm}, \code{npGSEAResultBeta}, 
      \code{npGSEAResultChiSq},
      \code{npGSEAResultNormCollection}, \code{npGSEAResultBetaCollection}, or
       \code{npGSEAResultChiSqCollection} }
}
\section{Methods}{
\describe{
    \item{\code{signature(object = "npGSEAResultNorm")}}{Returns a the name of 
    the gene set from a \code{npGSEAResultNorm} object}
    \item{\code{signature(object = "npGSEAResultBeta")}}{Returns a the name of the 
    gene set from a \code{npGSEAResultBeta} object}
    \item{\code{signature(object = "npGSEAResultChiSq")}}{Returns a the name of 
    the gene set from a \code{npGSEAResultChiSq} object} 
    \item{\code{signature(object = "npGSEAResultNormCollection")}}{Returns a list of 
    the names of the gene sets from a \code{npGSEAResultNormCollection} objects (1 for each set)} 
    \item{\code{signature(object = "npGSEAResultBetaCollection")}}{Returns a list of 
    the names of the gene sets from a \code{npGSEAResultBetaCollection} objects (1 for each set)}
   \item{\code{signature(object = "npGSEAResultChiSqCollection")}}{Returns a list of the 
   	names of the gene sets from a \code{npGSEAResultChiSqCollection} objects (1 for each set)}	
    }
}
\seealso{
  \code{\linkS4class{npGSEAResultNorm}}-class
}
\examples{
    set.seed(15)
    yFactor <- as.factor( c(rep("treated", 5), rep("control", 5)) )
    xData <- matrix(data=rnorm(length(letters)*10) ,nrow=length(letters), ncol=10)
    rownames(xData) <- letters
    geneSetABC15 <- GeneSet(geneIds=letters[1:15], setName="setABC15")
    res <- npGSEA(x = xData, y = yFactor, set = geneSetABC15)
    geneSetName (res)
}
\author{
Jessica L. Larson
}
\keyword{methods}

\name{pLeft-methods}
\docType{methods}
\alias{pLeft}
\alias{pLeft-methods}
\alias{pLeft,npGSEAResultNorm-method}
\alias{pLeft,npGSEAResultBeta-method}
\alias{pLeft,npGSEAResultNormCollection-method}
\alias{pLeft,npGSEAResultBetaCollection-method}
\title{ ~~ Methods for Function \code{pLeft}  ~~}
\description{
This function returns the left-sided p-value for the corresponding \code{GeneSet} 
or a list of p-values for a \code{GeneSetCollection}.  
This method is only applicable for the normal and beta approximation methods.
}
\usage{
pLeft(object)
}
\arguments{
\item{object}{An object of type \code{npGSEAResultNorm}, \code{npGSEAResultBeta},
    \code{npGSEAResultNormCollection}, or \code{npGSEAResultBetaCollection} }
}
\section{Methods}{
\describe{
    \item{\code{signature(object = "npGSEAResultNorm")}}{Returns a left-sided p-value 
    for a \code{npGSEAResultNorm} object}
    \item{\code{signature(object = "npGSEAResultBeta")}}{Returns a left-sided p-value 
    for a \code{npGSEAResultBeta} object}
    \item{\code{signature(object = "npGSEAResultNormCollection")}}{Returns a list of 
    left-sided p-values for a \code{npGSEAResultNormCollection} objects (1 for each set)}
    \item{\code{signature(object = "npGSEAResultBetaCollection")}}{Returns a list of 
    left-sided p-values for a \code{npGSEAResultBetaCollection} objects (1 for each set)}
}
}
\seealso{
    \code{\linkS4class{npGSEAResultNorm}}-class, \code{pRight}
}
\examples{
    set.seed(15)
    yFactor <- as.factor( c(rep("treated", 5), rep("control", 5)) )
    xData <- matrix(data=rnorm(length(letters)*10) ,nrow=length(letters), ncol=10)
    rownames(xData) <- letters
    geneSetABC15 <- GeneSet(geneIds=letters[1:15], setName="setABC15")
    res <- npGSEA(x = xData, y = yFactor, set = geneSetABC15)
    pLeft (res)
}
\author{
Jessica L. Larson
}
\keyword{methods}

\name{npGSEAResultChiSqCollection-class}
\Rdversion{1.1}
\docType{class}
\alias{npGSEAResultChiSqCollection-class}

\title{Class \code{"npGSEAResultChiSqCollection"}}
\description{
Objects of this class store results from running npGSEA with the Chi-square approximation with a GeneSetCollection.   
npGSEAResultChiSqCollection objects contain a list of npGSEAResultChiSq objects (one result for each GeneSet).
}
\section{Objects from the Class}{
Objects can be created by calls of \code{npGSEA}.
}
\author{
Jessica L. Larson
}

\seealso{
\code{\linkS4class{npGSEAResultNormCollection}}
}
\examples{
showClass("npGSEAResultChiSqCollection")
}
\keyword{classes}
\references{
Jessica L Larson and Art B Owen: Moment based gene set tests. BMC Bioinformatics 2015, 16:132. \url{http://www.biomedcentral.com/1471-2105/16/132}
}

\name{chiSqStat-methods}
\docType{methods}
\alias{chiSqStat}
\alias{chiSqStat-methods}
\alias{chiSqStat,npGSEAResultChiSq-method}
\alias{chiSqStat,npGSEAResultChiSqCollection-method}
\title{ ~~ Methods for Function \code{chiSqStat}  ~~}
\description{
This function returns the chi-sq statistic (which is compared to a reference 
Chi-sq distribution) for the chi-sq approximation of \code{npGSEA} for 
a corresponding \code{GeneSet} 
or a list of these statistics for a \code{GeneSetCollection}.  
This method is applicable for only the chi-sq approximation method.
}
\usage{chiSqStat(object)}
\arguments{
    \item{object}{An object of type 
        \code{npGSEAResultChiSq} or \code{npGSEAResultChiSqCollection} }
}
\section{Methods}{
\describe{
        \item{\code{signature(object = "npGSEAResultChiSq")}}{Returns the chi-sq statistic for a 
        \code{npGSEAResultChiSq} object}
        \item{\code{signature(object = "npGSEAResultChiSqCollection")}}{Returns a list of 
        the chi-sq statistics for a \code{npGSEAResultChiSqCollection} objects (1 for each set)}
        }
}
\seealso{\code{\linkS4class{npGSEAResultChiSq}}-class}
\examples{
    set.seed(15)
    yFactor <- as.factor( c(rep("treated", 5), rep("control", 5)) )
    xData <- matrix(data=rnorm(length(letters)*10) ,nrow=length(letters), ncol=10)
    rownames(xData) <- letters
    geneSetABC15 <- GeneSet(geneIds=letters[1:15], setName="setABC15")
    res <- npGSEA(x = xData, y = yFactor, set = geneSetABC15, approx= "chiSq")  
    chiSqStat(res)
}
\author{Jessica L. Larson}
\keyword{methods}

\name{npGSEAResultChiSq-class}
\Rdversion{1.1}
\docType{class}
\alias{npGSEAResultChiSq-class}

\title{Class \code{"npGSEAResultChiSq"}}
\description{
Objects of this class store results from running npGSEA with the Chi-square approximation. 
}
\section{Objects from the Class}{
Objects can be created by calls of \code{npGSEA}.
}
\section{Slots}{
    \describe{
        \item{\code{geneSetName}:}{Object of class \code{"character"}, the name of the geneSet }
        \item{\code{chiSqStat}:}{Object of class \code{"numeric"} , the test statistic, scaled to the corresponding chi-sq distribution }
        \item{\code{ChatGw}:}{Object of class \code{"numeric"}, the test statistic for the set }
        \item{\code{sigmaSq}:}{Object of class \code{"numeric"}, the variance }
        \item{\code{DF}:}{Object of class \code{"numeric"}, the degrees of freedom }
        \item{\code{pTwoSided}:}{Object of class \code{"numeric"}, the p-value for the two-sided hypothesis }
        \item{\code{xSet}:}{Object of class \code{"matrix"}, the centered and scaled x data for this set}
        \item{\code{betaHats}:}{Object of class \code{"vector"}, the betaHats for each gene in this set}
    }
}
\author{
Jessica L. Larson
}
\seealso{
\code{\linkS4class{npGSEAResultNorm}}
}
\examples{
showClass("npGSEAResultChiSq")
}
\keyword{classes}
\references{
Jessica L Larson and Art B Owen: Moment based gene set tests. BMC Bioinformatics 2015, 16:132. \url{http://www.biomedcentral.com/1471-2105/16/132}
}

\name{pValues-methods}
\docType{methods}
\alias{pValues}
\alias{pValues-methods}
\alias{pValues,npGSEAResultNorm-method}
\alias{pValues,npGSEAResultBeta-method}
\alias{pValues,npGSEAResultChiSq-method}
\title{ ~~ Methods for Function \code{pValues} ~~}
\description{
 ~~ Methods for function \code{pValues} ~~
}
\section{Methods}{
\describe{
    \item{
        \code{signature(x = "npGSEAResultNorm")},\code{signature(x = "npGSEAResultBeta")},
        \code{signature(x = "npGSEAResultChiSq")}}{These methods display the corresponding 
        p-values for the npGSEA analysis in the gene set in the given experiment.}
    }
}
\examples{
    set.seed(15)
    yFactor <- as.factor( c(rep("treated", 5), rep("control", 5)) )
    xData <- matrix(data=rnorm(length(letters)*10) ,nrow=length(letters), ncol=10)
    rownames(xData) <- letters
    geneSetABC15 <- GeneSet(geneIds=letters[1:15], setName="setABC15")
    res <- npGSEA(x = xData, y = yFactor, set = geneSetABC15)
    pValues(res)
}
\author{Jessica L. Larson}
\keyword{methods}

\name{zStat-methods}
\docType{methods}
\alias{zStat}
\alias{zStat-methods}
\alias{zStat,npGSEAResultNorm-method}
\alias{zStat,npGSEAResultNormCollection-method}
\title{ ~~ Methods for Function \code{zStat}  ~~}
\description{
This function returns the Z-statistic (which is compared to a reference 
standard normal distribution) for the normal approximation of \code{npGSEA} for 
a corresponding \code{GeneSet} 
or a list of these statistics for a \code{GeneSetCollection}.  
This method is applicable for only the normal approximation method.
}
\usage{
    zStat(object)
}
\arguments{
    \item{object}{An object of type 
        \code{npGSEAResultNorm} or \code{npGSEAResultNormCollection}}
}

\section{Methods}{
\describe{
    \item{\code{signature(object = "npGSEAResultNorm")}}{Returns the 
    Z-statistic for a \code{npGSEAResultNorm} object}
    \item{\code{signature(object = "npGSEAResultNormCollection")}}{Returns a 
    list of the Z- statistics for a \code{npGSEAResultNormCollection} objects (1 for each set)}
    }
}
\seealso{
    \code{\linkS4class{npGSEAResultNorm}}-class
}
\examples{
    set.seed(15)
    yFactor <- as.factor( c(rep("treated", 5), rep("control", 5)) )
    xData <- matrix(data=rnorm(length(letters)*10) ,nrow=length(letters), ncol=10)
    rownames(xData) <- letters
    geneSetABC15 <- GeneSet(geneIds=letters[1:15], setName="setABC15")
    res <- npGSEA(x = xData, y = yFactor, set = geneSetABC15, approx= "norm")  
    zStat(res)
}
\author{Jessica L. Larson}
\keyword{methods}

\name{DF-methods}
\docType{methods}
\alias{DF}
\alias{DF-methods}
\alias{DF,npGSEAResultChiSq-method}
\alias{DF,npGSEAResultChiSqCollection-method}
\title{ ~~ Methods for Function \code{DF}  ~~}
\description{
This function returns the degrees of freedom for the chi-sq approximation of a 
corresponding \code{GeneSet} 
or a list of degrees of freedom for a \code{GeneSetCollection}.  
This method is applicable for only the chi-sq approximation method.
}
\usage{DF(object)}
\arguments{
    \item{object}{An object of type 
        \code{npGSEAResultChiSq} or \code{npGSEAResultChiSqCollection} }
}
\section{Methods}{
\describe{
        \item{\code{signature(object = "npGSEAResultChiSq")}}{Returns the degrees of 
        freedom for a \code{npGSEAResultChiSq} object}
        \item{\code{signature(object = "npGSEAResultChiSqCollection")}}{Returns a 
        list of the degrees of freedom for a \code{npGSEAResultChiSqCollection} 
        objects (1 for each set)}
    }
}
\seealso{\code{\linkS4class{npGSEAResultChiSq}}-class}
\examples{
    set.seed(15)
    yFactor <- as.factor( c(rep("treated", 5), rep("control", 5)) )
    xData <- matrix(data=rnorm(length(letters)*10) ,nrow=length(letters), ncol=10)
    rownames(xData) <- letters
    geneSetABC15 <- GeneSet(geneIds=letters[1:15], setName="setABC15")
    res <- npGSEA(x = xData, y = yFactor, set = geneSetABC15, approx= "chiSq")  
    DF(res)
}
\author{Jessica L. Larson}
\keyword{methods}
